#ifndef __INCLUDE_DDK_STATUS_H
#define __INCLUDE_DDK_STATUS_H

#define NTSTAT_SEVERITY_SHIFT 30
#define NTSTAT_SEVERITY_MASK  0x00000003
#define NTSTAT_FACILITY_SHIFT 16
#define NTSTAT_FACILITY_MASK  0x00000FFF
#define NTSTAT_CUSTOMER_MASK  0x20000000

#define NT_SEVERITY(StatCode) (((StatCode) >> NTSTAT_SEVERITY_SHIFT) & NTSTAT_SEVERITY_MASK)
#define NT_FACILITY(StatCode) (((StatCode) >> NTSTAT_FACILITY_SHIFT) & NTSTAT_FACILITY_MASK)
#define NT_CUSTOMER(StatCode) ((StatCode) & NTSTAT_CUSTOMER_MASK)
#define NT_SUCCESS(StatCode)  ((NTSTATUS)(StatCode) >= 0)

/*
 * Possible status codes
 * FIXME: These may not be the actual values used by NT
 */

#define STATUS_SUCCESS                              (0x00000000)
#ifndef STATUS_WAIT_0
#define STATUS_WAIT_0                               (0x00000000)
#endif
#define STATUS_WAIT_63                              (0x0000003F)
#define STATUS_ABANDONED                            (0x00000080)
#ifndef STATUS_ABANDONED_WAIT_0
#define STATUS_ABANDONED_WAIT_0                     (0x00000080)
#endif
#define STATUS_ABANDONED_WAIT_63                    (0x000000BF)
#ifndef STATUS_USER_APC
#define STATUS_USER_APC                             (0x000000C0)
#endif
#define STATUS_KERNEL_APC                           (0x00000100)
#define STATUS_ALERTED                              (0x00000101)
#ifndef STATUS_TIMEOUT
#define STATUS_TIMEOUT                              (0x00000102)
#endif
#ifndef STATUS_PENDING
#define STATUS_PENDING                              (0x00000103)
#endif
#define STATUS_REPARSE                              (0x00000104)
#define STATUS_MORE_ENTRIES                         (0x00000105)
#define STATUS_NOT_ALL_ASSIGNED                     (0x00000106)
#define STATUS_SOME_NOT_MAPPED                      (0x00000107)
#define STATUS_OPLOCK_BREAK_IN_PROCESS              (0x00000108)
#define STATUS_VOLUME_MOUNTED                       (0x00000109)
#define STATUS_RXACT_COMMITTED                      (0x0000010A)
#define STATUS_NOTIFY_CLEANUP                       (0x0000010B)
#define STATUS_NOTIFY_ENUM_DIR                      (0x0000010C)
#define STATUS_NO_QUOTAS_NO_ACCOUNT                 (0x0000010D)
#define STATUS_PRIMARY_TRANSPORT_CONNECT_FAILED     (0x0000010E)

#define STATUS_OBJECT_EXISTS                        (0x40000000)
#define STATUS_THREAD_WAS_SUSPENDED                 (0x40000001)
#define STATUS_WORKING_SET_LIMIT_RANGE              (0x40000002)
#define STATUS_IMAGE_NOT_AT_BASE                    (0x40000003)
#define STATUS_RXACT_STATE_CREATED                  (0x40000004)
#define STATUS_SEGMENT_NOTIFICATION                 (0x40000005)
#define STATUS_LOCAL_USER_SESSION_KEY               (0x40000006)
#define STATUS_BAD_CURRENT_DIRECTORY                (0x40000007)
#define STATUS_SERIAL_MORE_WRITES                   (0x40000008)
#define STATUS_REGISTRY_RECOVERED                   (0x40000009)
#define STATUS_FT_READ_RECOVERING_FROM_BACKUP       (0x4000000A)
#define STATUS_FT_WRITE_RECOVERY                    (0x4000000B)
#define STATUS_SERIAL_COUNTER_TIMEOUT               (0x4000000C)
#define STATUS_NULL_LM_PASSWORD                     (0x4000000D)
#define STATUS_IMAGE_MACHINE_TYPE_MISMATCH          (0x4000000E)
#define STATUS_RECEIVE_PARTIAL                      (0x4000000F)
#define STATUS_RECEIVE_EXPEDITED                    (0x40000010)
#define STATUS_RECEIVE_PARTIAL_EXPEDITED            (0x40000011)
#define STATUS_EVENT_DONE                           (0x40000012)
#define STATUS_EVENT_PENDING                        (0x40000013)
#define STATUS_CHECKING_FILE_SYSTEM                 (0x40000014)
#define STATUS_FATAL_APP_EXIT                       (0x40000015)
#define STATUS_PREDEFINED_HANDLE                    (0x40000016)
#define STATUS_WAS_UNLOCKED                         (0x40000017)
#define STATUS_SERVICE_NOTIFICATION                 (0x40000018)
#define STATUS_WAS_LOCKED                           (0x40000019)
#define STATUS_LOG_HARD_ERROR                       (0x4000001A)
#define STATUS_ALREADY_WIN32                        (0x4000001B)
#define STATUS_WX86_UNSIMULATE                      (0x4000001C)
#define STATUS_WX86_CONTINUE                        (0x4000001D)
#define STATUS_WX86_SINGLE_STEP                     (0x4000001E)
#define STATUS_WX86_BREAKPOINT                      (0x4000001F)
#define STATUS_WX86_EXCEPTION_CONTINUE              (0x40000020)
#define STATUS_WX86_EXCEPTION_LASTCHANCE            (0x40000021)
#define STATUS_WX86_EXCEPTION_CHAIN                 (0x40000022)
#define STATUS_IMAGE_MACHINE_TYPE_MISMATCH_EXE      (0x40000023)
#define STATUS_NO_YIELD_PERFORMED                   (0x40000024)
#define STATUS_TIMER_RESUME_IGNORED                 (0x40000025)
#define STATUS_ARBITRATION_UNHANDLED                (0x40000026)
#define STATUS_CARDBUS_NOT_SUPPORTED                (0x40000027)
#define STATUS_WX86_CREATEWX86TIB                   (0x40000028)

#define STATUS_GUARD_PAGE_VIOLATION                 (0x80000001)
#define STATUS_DATATYPE_MISALIGNMENT                (0x80000002)
#define STATUS_BREAKPOINT                           (0x80000003)
#define STATUS_SINGLE_STEP                          (0x80000004)
#define STATUS_BUFFER_OVERFLOW                      (0x80000005)
#define STATUS_NO_MORE_FILES                        (0x80000006)
#define STATUS_WAKE_SYSTEM_DEBUGGER                 (0x80000007)

#define STATUS_HANDLES_CLOSED                       (0x8000000A)
#define STATUS_NO_INHERITANCE                       (0x8000000B)
#define STATUS_GUID_SUBSTITUTION_MADE               (0x8000000C)
#define STATUS_PARTIAL_COPY                         (0x8000000D)
#define STATUS_DEVICE_PAPER_EMPTY                   (0x8000000E)
#define STATUS_DEVICE_POWERED_OFF                   (0x8000000F)
#define STATUS_DEVICE_OFF_LINE                      (0x80000010)
#define STATUS_DEVICE_BUSY                          (0x80000011)
#define STATUS_NO_MORE_EAS                          (0x80000012)
#define STATUS_INVALID_EA_NAME                      (0x80000013)
#define STATUS_EA_LIST_INCONSISTENT                 (0x80000014)
#define STATUS_INVALID_EA_FLAG                      (0x80000015)
#define STATUS_VERIFY_REQUIRED                      (0x80000016)
#define STATUS_EXTRANEOUS_INFORMATION               (0x80000017)
#define STATUS_RXACT_COMMIT_NECESSARY               (0x80000018)

#define STATUS_NO_MORE_ENTRIES                      (0x8000001A)
#define STATUS_FILEMARK_DETECTED                    (0x8000001B)
#define STATUS_MEDIA_CHANGED                        (0x8000001C)
#define STATUS_BUS_RESET                            (0x8000001D)
#define STATUS_END_OF_MEDIA                         (0x8000001E)
#define STATUS_BEGINNING_OF_MEDIA                   (0x8000001F)
#define STATUS_MEDIA_CHECK                          (0x80000020)
#define STATUS_SETMARK_DETECTED                     (0x80000021)
#define STATUS_NO_DATA_DETECTED                     (0x80000022)
#define STATUS_REDIRECTOR_HAS_OPEN_HANDLES          (0x80000023)
#define STATUS_SERVER_HAS_OPEN_HANDLES              (0x80000024)
#define STATUS_ALREADY_DISCONNECTED                 (0x80000025)
#define STATUS_LONGJUMP                             (0x80000026)

#define STATUS_UNSUCCESSFUL                         (0xc0000001)
#define STATUS_NOT_IMPLEMENTED                      (0xc0000002)
#define STATUS_INVALID_INFO_CLASS                   (0xc0000003)
#define STATUS_INFO_LENGTH_MISMATCH                 (0xc0000004)
#define STATUS_ACCESS_VIOLATION                     (0xc0000005)
#define STATUS_IN_PAGE_ERROR                        (0xc0000006)
#define STATUS_PAGEFILE_QUOTA                       (0xc0000007)
#define STATUS_INVALID_HANDLE                       (0xc0000008)
#define STATUS_BAD_INITIAL_STACK                    (0xc0000009)
#define STATUS_BAD_INITIAL_PC                       (0xc000000a)
#define STATUS_INVALID_CID                          (0xc000000b)
#define STATUS_TIMER_NOT_CANCELED                   (0xc000000c)
#define STATUS_INVALID_PARAMETER                    (0xc000000d)
#define STATUS_NO_SUCH_DEVICE                       (0xc000000e)
#define STATUS_NO_SUCH_FILE                         (0xc000000f)
#define STATUS_INVALID_DEVICE_REQUEST               (0xc0000010)
#define STATUS_END_OF_FILE                          (0xc0000011)
#define STATUS_WRONG_VOLUME                         (0xc0000012)
#define STATUS_NO_MEDIA_IN_DEVICE                   (0xc0000013)
#define STATUS_UNRECOGNIZED_MEDIA                   (0xc0000014)
#define STATUS_NONEXISTENT_SECTOR                   (0xc0000015)
#define STATUS_MORE_PROCESSING_REQUIRED             (0xc0000016)
#define STATUS_NO_MEMORY                            (0xc0000017)
#define STATUS_CONFLICTING_ADDRESSES                (0xc0000018)
#define STATUS_NOT_MAPPED_VIEW                      (0xc0000019)
#define STATUS_UNABLE_TO_FREE_VM                    (0xc000001a)
#define STATUS_UNABLE_TO_DELETE_SECTION             (0xc000001b)
#define STATUS_INVALID_SYSTEM_SERVICE               (0xc000001c)
#define STATUS_ILLEGAL_INSTRUCTION                  (0xc000001d)
#define STATUS_INVALID_LOCK_SEQUENCE                (0xc000001e)
#define STATUS_INVALID_VIEW_SIZE                    (0xc000001f)
#define STATUS_INVALID_FILE_FOR_SECTION             (0xc0000020)
#define STATUS_ALREADY_COMMITTED                    (0xc0000021)
#define STATUS_ACCESS_DENIED                        (0xc0000022)
#define STATUS_BUFFER_TOO_SMALL                     (0xc0000023)
#define STATUS_OBJECT_TYPE_MISMATCH                 (0xc0000024)
#define STATUS_NONCONTINUABLE_EXCEPTION             (0xc0000025)
#define STATUS_INVALID_DISPOSITION                  (0xc0000026)
#define STATUS_UNWIND                               (0xc0000027)
#define STATUS_BAD_STACK                            (0xc0000028)
#define STATUS_INVALID_UNWIND_TARGET                (0xc0000029)
#define STATUS_NOT_LOCKED                           (0xc000002a)
#define STATUS_PARITY_ERROR                         (0xc000002b)
#define STATUS_UNABLE_TO_DECOMMIT_VM                (0xc000002c)
#define STATUS_NOT_COMMITTED                        (0xc000002d)
#define STATUS_INVALID_PORT_ATTRIBUTES              (0xc000002e)
#define STATUS_PORT_MESSAGE_TOO_LONG                (0xc000002f)
#define STATUS_INVALID_PARAMETER_MIX                (0xc0000030)
#define STATUS_INVALID_QUOTA_LOWER                  (0xc0000031)
#define STATUS_DISK_CORRUPT_ERROR                   (0xc0000032)
#define STATUS_OBJECT_NAME_INVALID                  (0xc0000033)
#define STATUS_OBJECT_NAME_NOT_FOUND                (0xc0000034)
#define STATUS_OBJECT_NAME_COLLISION                (0xc0000035)

#define STATUS_PORT_DISCONNECTED                    (0xc0000037)
#define STATUS_DEVICE_ALREADY_ATTACHED              (0xc0000038)
#define STATUS_OBJECT_PATH_INVALID                  (0xc0000039)
#define STATUS_OBJECT_PATH_NOT_FOUND                (0xc000003a)
#define STATUS_PATH_SYNTAX_BAD                      (0xc000003b)
#define STATUS_DATA_OVERRUN                         (0xc000003c)
#define STATUS_DATA_LATE_ERROR                      (0xc000003d)
#define STATUS_DATA_ERROR                           (0xc000003e)
#define STATUS_CRC_ERROR                            (0xc000003f)
#define STATUS_SECTION_TOO_BIG                      (0xc0000040)
#define STATUS_PORT_CONNECTION_REFUSED              (0xc0000041)
#define STATUS_INVALID_PORT_HANDLE                  (0xc0000042)
#define STATUS_SHARING_VIOLATION                    (0xc0000043)
#define STATUS_QUOTA_EXCEEDED                       (0xc0000044)
#define STATUS_INVALID_PAGE_PROTECTION              (0xc0000045)
#define STATUS_MUTANT_NOT_OWNED                     (0xc0000046)
#define STATUS_SEMAPHORE_LIMIT_EXCEEDED             (0xc0000047)
#define STATUS_PORT_ALREADY_SET                     (0xc0000048)
#define STATUS_SECTION_NOT_IMAGE                    (0xc0000049)
#define STATUS_SUSPEND_COUNT_EXCEEDED               (0xc000004a)
#define STATUS_THREAD_IS_TERMINATING                (0xc000004b)
#define STATUS_BAD_WORKING_SET_LIMIT                (0xc000004c)
#define STATUS_INCOMPATIBLE_FILE_MAP                (0xc000004d)
#define STATUS_SECTION_PROTECTION                   (0xc000004e)
#define STATUS_EAS_NOT_SUPPORTED                    (0xc000004f)
#define STATUS_EA_TOO_LARGE                         (0xc0000050)
#define STATUS_NONEXISTENT_EA_ENTRY                 (0xc0000051)
#define STATUS_NO_EAS_ON_FILE                       (0xc0000052)
#define STATUS_EA_CORRUPT_ERROR                     (0xc0000053)
#define STATUS_FILE_LOCK_CONFLICT                   (0xc0000054)
#define STATUS_LOCK_NOT_GRANTED                     (0xc0000055)
#define STATUS_DELETE_PENDING                       (0xc0000056)
#define STATUS_CTL_FILE_NOT_SUPPORTED               (0xc0000057)
#define STATUS_UNKNOWN_REVISION                     (0xc0000058)
#define STATUS_REVISION_MISMATCH                    (0xc0000059)
#define STATUS_INVALID_OWNER                        (0xc000005a)
#define STATUS_INVALID_PRIMARY_GROUP                (0xc000005b)
#define STATUS_NO_IMPERSONATION_TOKEN               (0xc000005c)
#define STATUS_CANT_DISABLE_MANDATORY               (0xc000005d)
#define STATUS_NO_LOGON_SERVERS                     (0xc000005e)
#define STATUS_NO_SUCH_LOGON_SESSION                (0xc000005f)
#define STATUS_NO_SUCH_PRIVILEGE                    (0xc0000060)
#define STATUS_PRIVILEGE_NOT_HELD                   (0xc0000061)
#define STATUS_INVALID_ACCOUNT_NAME                 (0xc0000062)
#define STATUS_USER_EXISTS                          (0xc0000063)
#define STATUS_NO_SUCH_USER                         (0xc0000064)
#define STATUS_GROUP_EXISTS                         (0xc0000065)
#define STATUS_NO_SUCH_GROUP                        (0xc0000066)
#define STATUS_MEMBER_IN_GROUP                      (0xc0000067)
#define STATUS_MEMBER_NOT_IN_GROUP                  (0xc0000068)
#define STATUS_LAST_ADMIN                           (0xc0000069)
#define STATUS_WRONG_PASSWORD                       (0xc000006a)
#define STATUS_ILL_FORMED_PASSWORD                  (0xc000006b)
#define STATUS_PASSWORD_RESTRICTION                 (0xc000006c)
#define STATUS_LOGON_FAILURE                        (0xc000006d)
#define STATUS_ACCOUNT_RESTRICTION                  (0xc000006e)
#define STATUS_INVALID_LOGON_HOURS                  (0xc000006f)
#define STATUS_INVALID_WORKSTATION                  (0xc0000070)
#define STATUS_PASSWORD_EXPIRED                     (0xc0000071)
#define STATUS_ACCOUNT_DISABLED                     (0xc0000072)
#define STATUS_NONE_MAPPED                          (0xc0000073)
#define STATUS_TOO_MANY_LUIDS_REQUESTED             (0xc0000074)
#define STATUS_LUIDS_EXHAUSTED                      (0xc0000075)
#define STATUS_INVALID_SUB_AUTHORITY                (0xc0000076)
#define STATUS_INVALID_ACL                          (0xc0000077)
#define STATUS_INVALID_SID                          (0xc0000078)
#define STATUS_INVALID_SECURITY_DESCR               (0xc0000079)
#define STATUS_PROCEDURE_NOT_FOUND                  (0xc000007a)
#define STATUS_INVALID_IMAGE_FORMAT                 (0xc000007b)
#define STATUS_NO_TOKEN                             (0xc000007c)
#define STATUS_BAD_INHERITANCE_ACL                  (0xc000007d)
#define STATUS_RANGE_NOT_LOCKED                     (0xc000007e)
#define STATUS_DISK_FULL                            (0xc000007f)
#define STATUS_SERVER_DISABLED                      (0xc0000080)
#define STATUS_SERVER_NOT_DISABLED                  (0xc0000081)
#define STATUS_TOO_MANY_GUIDS_REQUESTED             (0xc0000082)
#define STATUS_GUIDS_EXHAUSTED                      (0xc0000083)
#define STATUS_INVALID_ID_AUTHORITY                 (0xc0000084)
#define STATUS_AGENTS_EXHAUSTED                     (0xc0000085)
#define STATUS_INVALID_VOLUME_LABEL                 (0xc0000086)
#define STATUS_SECTION_NOT_EXTENDED                 (0xc0000087)
#define STATUS_NOT_MAPPED_DATA                      (0xc0000088)
#define STATUS_RESOURCE_DATA_NOT_FOUND              (0xc0000089)
#define STATUS_RESOURCE_TYPE_NOT_FOUND              (0xc000008a)
#define STATUS_RESOURCE_NAME_NOT_FOUND              (0xc000008b)
#define STATUS_ARRAY_BOUNDS_EXCEEDED                (0xc000008c)
#define STATUS_FLOAT_DENORMAL_OPERAND               (0xc000008d)
#define STATUS_FLOAT_DIVIDE_BY_ZERO                 (0xc000008e)
#define STATUS_FLOAT_INEXACT_RESULT                 (0xc000008f)
#define STATUS_FLOAT_INVALID_OPERATION              (0xc0000090)
#define STATUS_FLOAT_OVERFLOW                       (0xc0000091)
#define STATUS_FLOAT_STACK_CHECK                    (0xc0000092)
#define STATUS_FLOAT_UNDERFLOW                      (0xc0000093)
#define STATUS_INTEGER_DIVIDE_BY_ZERO               (0xc0000094)
#define STATUS_INTEGER_OVERFLOW                     (0xc0000095)
#define STATUS_PRIVILEGED_INSTRUCTION               (0xc0000096)
#define STATUS_TOO_MANY_PAGING_FILES                (0xc0000097)
#define STATUS_FILE_INVALID                         (0xc0000098)
#define STATUS_ALLOTTED_SPACE_EXCEEDED              (0xc0000099)
#define STATUS_INSUFFICIENT_RESOURCES               (0xc000009a)
#define STATUS_DFS_EXIT_PATH_FOUND                  (0xc000009b)
#define STATUS_DEVICE_DATA_ERROR                    (0xc000009c)
#define STATUS_DEVICE_NOT_CONNECTED                 (0xc000009d)
#define STATUS_DEVICE_POWER_FAILURE                 (0xc000009e)
#define STATUS_FREE_VM_NOT_AT_BASE                  (0xc000009f)
#define STATUS_MEMORY_NOT_ALLOCATED                 (0xc00000a0)
#define STATUS_WORKING_SET_QUOTA                    (0xc00000a1)
#define STATUS_MEDIA_WRITE_PROTECTED                (0xc00000a2)
#define STATUS_DEVICE_NOT_READY                     (0xc00000a3)
#define STATUS_INVALID_GROUP_ATTRIBUTES             (0xc00000a4)
#define STATUS_BAD_IMPERSONATION_LEVEL              (0xc00000a5)
#define STATUS_CANT_OPEN_ANONYMOUS                  (0xc00000a6)
#define STATUS_BAD_VALIDATION_CLASS                 (0xc00000a7)
#define STATUS_BAD_TOKEN_TYPE                       (0xc00000a8)
#define STATUS_BAD_MASTER_BOOT_RECORD               (0xc00000a9)
#define STATUS_INSTRUCTION_MISALIGNMENT             (0xc00000aa)
#define STATUS_INSTANCE_NOT_AVAILABLE               (0xc00000ab)
#define STATUS_PIPE_NOT_AVAILABLE                   (0xc00000ac)
#define STATUS_INVALID_PIPE_STATE                   (0xc00000ad)
#define STATUS_PIPE_BUSY                            (0xc00000ae)
#define STATUS_ILLEGAL_FUNCTION                     (0xc00000af)
#define STATUS_PIPE_DISCONNECTED                    (0xc00000b0)
#define STATUS_PIPE_CLOSING                         (0xc00000b1)
#define STATUS_PIPE_CONNECTED                       (0xc00000b2)
#define STATUS_PIPE_LISTENING                       (0xc00000b3)
#define STATUS_INVALID_READ_MODE                    (0xc00000b4)
#define STATUS_IO_TIMEOUT                           (0xc00000b5)
#define STATUS_FILE_FORCED_CLOSED                   (0xc00000b6)
#define STATUS_PROFILING_NOT_STARTED                (0xc00000b7)
#define STATUS_PROFILING_NOT_STOPPED                (0xc00000b8)
#define STATUS_COULD_NOT_INTERPRET                  (0xc00000b9)
#define STATUS_FILE_IS_A_DIRECTORY                  (0xc00000ba)
#define STATUS_NOT_SUPPORTED                        (0xc00000bb)
#define STATUS_REMOTE_NOT_LISTENING                 (0xc00000bc)
#define STATUS_DUPLICATE_NAME                       (0xc00000bd)
#define STATUS_BAD_NETWORK_PATH                     (0xc00000be)
#define STATUS_NETWORK_BUSY                         (0xc00000bf)
#define STATUS_DEVICE_DOES_NOT_EXIST                (0xc00000c0)
#define STATUS_TOO_MANY_COMMANDS                    (0xc00000c1)
#define STATUS_ADAPTER_HARDWARE_ERROR               (0xc00000c2)
#define STATUS_INVALID_NETWORK_RESPONSE             (0xc00000c3)
#define STATUS_UNEXPECTED_NETWORK_ERROR             (0xc00000c4)
#define STATUS_BAD_REMOTE_ADAPTER                   (0xc00000c5)
#define STATUS_PRINT_QUEUE_FULL                     (0xc00000c6)
#define STATUS_NO_SPOOL_SPACE                       (0xc00000c7)
#define STATUS_PRINT_CANCELLED                      (0xc00000c8)
#define STATUS_NETWORK_NAME_DELETED                 (0xc00000c9)
#define STATUS_NETWORK_ACCESS_DENIED                (0xc00000ca)
#define STATUS_BAD_DEVICE_TYPE                      (0xc00000cb)
#define STATUS_BAD_NETWORK_NAME                     (0xc00000cc)
#define STATUS_TOO_MANY_NAMES                       (0xc00000cd)
#define STATUS_TOO_MANY_SESSIONS                    (0xc00000ce)
#define STATUS_SHARING_PAUSED                       (0xc00000cf)
#define STATUS_REQUEST_NOT_ACCEPTED                 (0xc00000d0)
#define STATUS_REDIRECTOR_PAUSED                    (0xc00000d1)
#define STATUS_NET_WRITE_FAULT                      (0xc00000d2)
#define STATUS_PROFILING_AT_LIMIT                   (0xc00000d3)
#define STATUS_NOT_SAME_DEVICE                      (0xc00000d4)
#define STATUS_FILE_RENAMED                         (0xc00000d5)
#define STATUS_VIRTUAL_CIRCUIT_CLOSED               (0xc00000d6)
#define STATUS_NO_SECURITY_ON_OBJECT                (0xc00000d7)
#define STATUS_CANT_WAIT                            (0xc00000d8)
#define STATUS_PIPE_EMPTY                           (0xc00000d9)
#define STATUS_CANT_ACCESS_DOMAIN_INFO              (0xc00000da)
#define STATUS_CANT_TERMINATE_SELF                  (0xc00000db)
#define STATUS_INVALID_SERVER_STATE                 (0xc00000dc)
#define STATUS_INVALID_DOMAIN_STATE                 (0xc00000dd)
#define STATUS_INVALID_DOMAIN_ROLE                  (0xc00000de)
#define STATUS_NO_SUCH_DOMAIN                       (0xc00000df)
#define STATUS_DOMAIN_EXISTS                        (0xc00000e0)
#define STATUS_DOMAIN_LIMIT_EXCEEDED                (0xc00000e1)
#define STATUS_OPLOCK_NOT_GRANTED                   (0xc00000e2)
#define STATUS_INVALID_OPLOCK_PROTOCOL              (0xc00000e3)
#define STATUS_INTERNAL_DB_CORRUPTION               (0xc00000e4)
#define STATUS_INTERNAL_ERROR                       (0xc00000e5)
#define STATUS_GENERIC_NOT_MAPPED                   (0xc00000e6)
#define STATUS_BAD_DESCRIPTOR_FORMAT                (0xc00000e7)
#define STATUS_INVALID_USER_BUFFER                  (0xc00000e8)
#define STATUS_UNEXPECTED_IO_ERROR                  (0xc00000e9)
#define STATUS_UNEXPECTED_MM_CREATE_ERR             (0xc00000ea)
#define STATUS_UNEXPECTED_MM_MAP_ERROR              (0xc00000eb)
#define STATUS_UNEXPECTED_MM_EXTEND_ERR             (0xc00000ec)
#define STATUS_NOT_LOGON_PROCESS                    (0xc00000ed)
#define STATUS_LOGON_SESSION_EXISTS                 (0xc00000ee)
#define STATUS_INVALID_PARAMETER_1                  (0xc00000ef)
#define STATUS_INVALID_PARAMETER_2                  (0xc00000f0)
#define STATUS_INVALID_PARAMETER_3                  (0xc00000f1)
#define STATUS_INVALID_PARAMETER_4                  (0xc00000f2)
#define STATUS_INVALID_PARAMETER_5                  (0xc00000f3)
#define STATUS_INVALID_PARAMETER_6                  (0xc00000f4)
#define STATUS_INVALID_PARAMETER_7                  (0xc00000f5)
#define STATUS_INVALID_PARAMETER_8                  (0xc00000f6)
#define STATUS_INVALID_PARAMETER_9                  (0xc00000f7)
#define STATUS_INVALID_PARAMETER_10                 (0xc00000f8)
#define STATUS_INVALID_PARAMETER_11                 (0xc00000f9)
#define STATUS_INVALID_PARAMETER_12                 (0xc00000fa)
#define STATUS_REDIRECTOR_NOT_STARTED               (0xc00000fb)
#define STATUS_REDIRECTOR_STARTED                   (0xc00000fc)
#define STATUS_STACK_OVERFLOW                       (0xc00000fd)
#define STATUS_NO_SUCH_PACKAGE                      (0xc00000fe)
#define STATUS_BAD_FUNCTION_TABLE                   (0xc00000ff)
#define STATUS_VARIABLE_NOT_FOUND                   (0xc0000100)
#define STATUS_DIRECTORY_NOT_EMPTY                  (0xc0000101)
#define STATUS_FILE_CORRUPT_ERROR                   (0xc0000102)
#define STATUS_NOT_A_DIRECTORY                      (0xc0000103)
#define STATUS_BAD_LOGON_SESSION_STATE              (0xc0000104)
#define STATUS_LOGON_SESSION_COLLISION              (0xc0000105)
#define STATUS_NAME_TOO_LONG                        (0xc0000106)
#define STATUS_FILES_OPEN                           (0xc0000107)
#define STATUS_CONNECTION_IN_USE                    (0xc0000108)
#define STATUS_MESSAGE_NOT_FOUND                    (0xc0000109)
#define STATUS_PROCESS_IS_TERMINATING               (0xc000010a)
#define STATUS_INVALID_LOGON_TYPE                   (0xc000010b)
#define STATUS_NO_GUID_TRANSLATION                  (0xc000010c)
#define STATUS_CANNOT_IMPERSONATE                   (0xc000010d)
#define STATUS_IMAGE_ALREADY_LOADED                 (0xc000010e)
#define STATUS_ABIOS_NOT_PRESENT                    (0xc000010f)
#define STATUS_ABIOS_LID_NOT_EXIST                  (0xc0000110)
#define STATUS_ABIOS_LID_ALREADY_OWNED              (0xc0000111)
#define STATUS_ABIOS_NOT_LID_OWNER                  (0xc0000112)
#define STATUS_ABIOS_INVALID_COMMAND                (0xc0000113)
#define STATUS_ABIOS_INVALID_LID                    (0xc0000114)
#define STATUS_ABIOS_SELECTOR_NOT_AVAILABLE         (0xc0000115)
#define STATUS_ABIOS_INVALID_SELECTOR               (0xc0000116)
#define STATUS_NO_LDT                               (0xc0000117)
#define STATUS_INVALID_LDT_SIZE                     (0xc0000118)
#define STATUS_INVALID_LDT_OFFSET                   (0xc0000119)
#define STATUS_INVALID_LDT_DESCRIPTOR               (0xc000011a)
#define STATUS_INVALID_IMAGE_NE_FORMAT              (0xc000011b)
#define STATUS_RXACT_INVALID_STATE                  (0xc000011c)
#define STATUS_RXACT_COMMIT_FAILURE                 (0xc000011d)
#define STATUS_MAPPED_FILE_SIZE_ZERO                (0xc000011e)
#define STATUS_TOO_MANY_OPENED_FILES                (0xc000011f)
#define STATUS_CANCELLED                            (0xc0000120)
#define STATUS_CANNOT_DELETE                        (0xc0000121)
#define STATUS_INVALID_COMPUTER_NAME                (0xc0000122)
#define STATUS_FILE_DELETED                         (0xc0000123)
#define STATUS_SPECIAL_ACCOUNT                      (0xc0000124)
#define STATUS_SPECIAL_GROUP                        (0xc0000125)
#define STATUS_SPECIAL_USER                         (0xc0000126)
#define STATUS_MEMBERS_PRIMARY_GROUP                (0xc0000127)
#define STATUS_FILE_CLOSED                          (0xc0000128)
#define STATUS_TOO_MANY_THREADS                     (0xc0000129)
#define STATUS_THREAD_NOT_IN_PROCESS                (0xc000012a)
#define STATUS_TOKEN_ALREADY_IN_USE                 (0xc000012b)
#define STATUS_PAGEFILE_QUOTA_EXCEEDED              (0xc000012c)
#define STATUS_COMMITMENT_LIMIT                     (0xc000012d)
#define STATUS_INVALID_IMAGE_LE_FORMAT              (0xc000012e)
#define STATUS_INVALID_IMAGE_NOT_MZ                 (0xc000012f)
#define STATUS_INVALID_IMAGE_PROTECT                (0xc0000130)
#define STATUS_INVALID_IMAGE_WIN_16                 (0xc0000131)
#define STATUS_LOGON_SERVER_CONFLICT                (0xc0000132)
#define STATUS_TIME_DIFFERENCE_AT_DC                (0xc0000133)
#define STATUS_SYNCHRONIZATION_REQUIRED             (0xc0000134)
#define STATUS_DLL_NOT_FOUND                        (0xc0000135)
#define STATUS_OPEN_FAILED                          (0xc0000136)
#define STATUS_IO_PRIVILEGE_FAILED                  (0xc0000137)
#define STATUS_ORDINAL_NOT_FOUND                    (0xc0000138)
#define STATUS_ENTRYPOINT_NOT_FOUND                 (0xc0000139)
#define STATUS_CONTROL_C_EXIT                       (0xc000013a)
#define STATUS_LOCAL_DISCONNECT                     (0xc000013b)
#define STATUS_REMOTE_DISCONNECT                    (0xc000013c)
#define STATUS_REMOTE_RESOURCES                     (0xc000013d)
#define STATUS_LINK_FAILED                          (0xc000013e)
#define STATUS_LINK_TIMEOUT                         (0xc000013f)
#define STATUS_INVALID_CONNECTION                   (0xc0000140)
#define STATUS_INVALID_ADDRESS                      (0xc0000141)
#define STATUS_DLL_INIT_FAILED                      (0xc0000142)
#define STATUS_MISSING_SYSTEMFILE                   (0xc0000143)
#define STATUS_UNHANDLED_EXCEPTION                  (0xc0000144)
#define STATUS_APP_INIT_FAILURE                     (0xc0000145)
#define STATUS_PAGEFILE_CREATE_FAILED               (0xc0000146)
#define STATUS_NO_PAGEFILE                          (0xc0000147)
#define STATUS_INVALID_LEVEL                        (0xc0000148)
#define STATUS_WRONG_PASSWORD_CORE                  (0xc0000149)
#define STATUS_ILLEGAL_FLOAT_CONTEXT                (0xc000014a)
#define STATUS_PIPE_BROKEN                          (0xc000014b)
#define STATUS_REGISTRY_CORRUPT                     (0xc000014c)
#define STATUS_REGISTRY_IO_FAILED                   (0xc000014d)
#define STATUS_NO_EVENT_PAIR                        (0xc000014e)
#define STATUS_UNRECOGNIZED_VOLUME                  (0xc000014f)
#define STATUS_SERIAL_NO_DEVICE_INITED              (0xc0000150)
#define STATUS_NO_SUCH_ALIAS                        (0xc0000151)
#define STATUS_MEMBER_NOT_IN_ALIAS                  (0xc0000152)
#define STATUS_MEMBER_IN_ALIAS                      (0xc0000153)
#define STATUS_ALIAS_EXISTS                         (0xc0000154)
#define STATUS_LOGON_NOT_GRANTED                    (0xc0000155)
#define STATUS_TOO_MANY_SECRETS                     (0xc0000156)
#define STATUS_SECRET_TOO_LONG                      (0xc0000157)
#define STATUS_INTERNAL_DB_ERROR                    (0xc0000158)
#define STATUS_FULLSCREEN_MODE                      (0xc0000159)
#define STATUS_TOO_MANY_CONTEXT_IDS                 (0xc000015a)
#define STATUS_LOGON_TYPE_NOT_GRANTED               (0xc000015b)
#define STATUS_NOT_REGISTRY_FILE                    (0xc000015c)
#define STATUS_NT_CROSS_ENCRYPTION_REQUIRED         (0xc000015d)
#define STATUS_DOMAIN_CTRLR_CONFIG_ERROR            (0xc000015e)
#define STATUS_FT_MISSING_MEMBER                    (0xc000015f)
#define STATUS_ILL_FORMED_SERVICE_ENTRY             (0xc0000160)
#define STATUS_ILLEGAL_CHARACTER                    (0xc0000161)
#define STATUS_UNMAPPABLE_CHARACTER                 (0xc0000162)
#define STATUS_UNDEFINED_CHARACTER                  (0xc0000163)
#define STATUS_FLOPPY_VOLUME                        (0xc0000164)
#define STATUS_FLOPPY_ID_MARK_NOT_FOUND             (0xc0000165)
#define STATUS_FLOPPY_WRONG_CYLINDER                (0xc0000166)
#define STATUS_FLOPPY_UNKNOWN_ERROR                 (0xc0000167)
#define STATUS_FLOPPY_BAD_REGISTERS                 (0xc0000168)
#define STATUS_DISK_RECALIBRATE_FAILED              (0xc0000169)
#define STATUS_DISK_OPERATION_FAILED                (0xc000016a)
#define STATUS_DISK_RESET_FAILED                    (0xc000016b)
#define STATUS_SHARED_IRQ_BUSY                      (0xc000016c)
#define STATUS_FT_ORPHANING                         (0xc000016d)
#define STATUS_BIOS_FAILED_TO_CONNECT_INTERRUPT     (0xc000016e)

#define STATUS_PARTITION_FAILURE                    (0xc0000172)
#define STATUS_INVALID_BLOCK_LENGTH                 (0xc0000173)
#define STATUS_DEVICE_NOT_PARTITIONED               (0xc0000174)
#define STATUS_UNABLE_TO_LOCK_MEDIA                 (0xc0000175)
#define STATUS_UNABLE_TO_UNLOAD_MEDIA               (0xc0000176)
#define STATUS_EOM_OVERFLOW                         (0xc0000177)
#define STATUS_NO_MEDIA                             (0xc0000178)

#define STATUS_NO_SUCH_MEMBER                       (0xc000017a)
#define STATUS_INVALID_MEMBER                       (0xc000017b)
#define STATUS_KEY_DELETED                          (0xc000017c)
#define STATUS_NO_LOG_SPACE                         (0xc000017d)
#define STATUS_TOO_MANY_SIDS                        (0xc000017e)
#define STATUS_LM_CROSS_ENCRYPTION_REQUIRED         (0xc000017f)
#define STATUS_KEY_HAS_CHILDREN                     (0xc0000180)
#define STATUS_CHILD_MUST_BE_VOLATILE               (0xc0000181)
#define STATUS_DEVICE_CONFIGURATION_ERROR           (0xc0000182)
#define STATUS_DRIVER_INTERNAL_ERROR                (0xc0000183)
#define STATUS_INVALID_DEVICE_STATE                 (0xc0000184)
#define STATUS_IO_DEVICE_ERROR                      (0xc0000185)
#define STATUS_DEVICE_PROTOCOL_ERROR                (0xc0000186)
#define STATUS_BACKUP_CONTROLLER                    (0xc0000187)
#define STATUS_LOG_FILE_FULL                        (0xc0000188)
#define STATUS_TOO_LATE                             (0xc0000189)
#define STATUS_NO_TRUST_LSA_SECRET                  (0xc000018a)
#define STATUS_NO_TRUST_SAM_ACCOUNT                 (0xc000018b)
#define STATUS_TRUSTED_DOMAIN_FAILURE               (0xc000018c)
#define STATUS_TRUSTED_RELATIONSHIP_FAILURE         (0xc000018d)
#define STATUS_EVENTLOG_FILE_CORRUPT                (0xc000018e)
#define STATUS_EVENTLOG_CANT_START                  (0xc000018f)
#define STATUS_TRUST_FAILURE                        (0xc0000190)
#define STATUS_MUTANT_LIMIT_EXCEEDED                (0xc0000191)
#define STATUS_NETLOGON_NOT_STARTED                 (0xc0000192)
#define STATUS_ACCOUNT_EXPIRED                      (0xc0000193)
#define STATUS_POSSIBLE_DEADLOCK                    (0xc0000194)
#define STATUS_NETWORK_CREDENTIAL_CONFLICT          (0xc0000195)
#define STATUS_REMOTE_SESSION_LIMIT                 (0xc0000196)
#define STATUS_EVENTLOG_FILE_CHANGED                (0xc0000197)
#define STATUS_NOLOGON_INTERDOMAIN_TRUST_ACCOUNT    (0xc0000198)
#define STATUS_NOLOGON_WORKSTATION_TRUST_ACCOUNT    (0xc0000199)
#define STATUS_NOLOGON_SERVER_TRUST_ACCOUNT         (0xc000019a)
#define STATUS_DOMAIN_TRUST_INCONSISTENT            (0xc000019b)
#define STATUS_FS_DRIVER_REQUIRED                   (0xc000019c)

#define STATUS_NO_USER_SESSION_KEY                  (0xc0000202)
#define STATUS_USER_SESSION_DELETED                 (0xc0000203)
#define STATUS_RESOURCE_LANG_NOT_FOUND              (0xc0000204)
#define STATUS_INSUFF_SERVER_RESOURCES              (0xc0000205)
#define STATUS_INVALID_BUFFER_SIZE                  (0xc0000206)
#define STATUS_INVALID_ADDRESS_COMPONENT            (0xc0000207)
#define STATUS_INVALID_ADDRESS_WILDCARD             (0xc0000208)
#define STATUS_TOO_MANY_ADDRESSES                   (0xc0000209)
#define STATUS_ADDRESS_ALREADY_EXISTS               (0xc000020a)
#define STATUS_ADDRESS_CLOSED                       (0xc000020b)
#define STATUS_CONNECTION_DISCONNECTED              (0xc000020c)
#define STATUS_CONNECTION_RESET                     (0xc000020d)
#define STATUS_TOO_MANY_NODES                       (0xc000020e)
#define STATUS_TRANSACTION_ABORTED                  (0xc000020f)
#define STATUS_TRANSACTION_TIMED_OUT                (0xc0000210)
#define STATUS_TRANSACTION_NO_RELEASE               (0xc0000211)
#define STATUS_TRANSACTION_NO_MATCH                 (0xc0000212)
#define STATUS_TRANSACTION_RESPONDED                (0xc0000213)
#define STATUS_TRANSACTION_INVALID_ID               (0xc0000214)
#define STATUS_TRANSACTION_INVALID_TYPE             (0xc0000215)
#define STATUS_NOT_SERVER_SESSION                   (0xc0000216)
#define STATUS_NOT_CLIENT_SESSION                   (0xc0000217)
#define STATUS_CANNOT_LOAD_REGISTRY_FILE            (0xc0000218)
#define STATUS_DEBUG_ATTACH_FAILED                  (0xc0000219)
#define STATUS_SYSTEM_PROCESS_TERMINATED            (0xc000021a)
#define STATUS_DATA_NOT_ACCEPTED                    (0xc000021b)
#define STATUS_NO_BROWSER_SERVERS_FOUND             (0xc000021c)
#define STATUS_VDM_HARD_ERROR                       (0xc000021d)
#define STATUS_DRIVER_CANCEL_TIMEOUT                (0xc000021e)
#define STATUS_REPLY_MESSAGE_MISMATCH               (0xc000021f)
#define STATUS_MAPPED_ALIGNMENT                     (0xc0000220)
#define STATUS_IMAGE_CHECKSUM_MISMATCH              (0xc0000221)
#define STATUS_LOST_WRITEBEHIND_DATA                (0xc0000222)
#define STATUS_CLIENT_SERVER_PARAMETERS_INVALID     (0xc0000223)
#define STATUS_PASSWORD_MUST_CHANGE                 (0xc0000224)
#define STATUS_NOT_FOUND                            (0xc0000225)
#define STATUS_NOT_TINY_STREAM                      (0xc0000226)
#define STATUS_RECOVERY_FAILURE                     (0xc0000227)
#define STATUS_STACK_OVERFLOW_READ                  (0xc0000228)
#define STATUS_FAIL_CHECK                           (0xc0000229)
#define STATUS_DUPLICATE_OBJECTID                   (0xc000022a)
#define STATUS_OBJECTID_EXISTS                      (0xc000022b)
#define STATUS_CONVERT_TO_LARGE                     (0xc000022c)
#define STATUS_RETRY                                (0xc000022d)
#define STATUS_FOUND_OUT_OF_SCOPE                   (0xc000022e)
#define STATUS_ALLOCATE_BUCKET                      (0xc000022f)
#define STATUS_PROPSET_NOT_FOUND                    (0xc0000230)
#define STATUS_MARSHALL_OVERFLOW                    (0xc0000231)
#define STATUS_INVALID_VARIANT                      (0xc0000232)
#define STATUS_DOMAIN_CONTROLLER_NOT_FOUND          (0xc0000233)
#define STATUS_ACCOUNT_LOCKED_OUT                   (0xc0000234)
#define STATUS_HANDLE_NOT_CLOSABLE                  (0xc0000235)
#define STATUS_CONNECTION_REFUSED                   (0xc0000236)
#define STATUS_GRACEFUL_DISCONNECT                  (0xc0000237)
#define STATUS_ADDRESS_ALREADY_ASSOCIATED           (0xc0000238)
#define STATUS_ADDRESS_NOT_ASSOCIATED               (0xc0000239)
#define STATUS_CONNECTION_INVALID                   (0xc000023a)
#define STATUS_CONNECTION_ACTIVE                    (0xc000023b)
#define STATUS_NETWORK_UNREACHABLE                  (0xc000023c)
#define STATUS_HOST_UNREACHABLE                     (0xc000023d)
#define STATUS_PROTOCOL_UNREACHABLE                 (0xc000023e)
#define STATUS_PORT_UNREACHABLE                     (0xc000023f)
#define STATUS_REQUEST_ABORTED                      (0xc0000240)
#define STATUS_CONNECTION_ABORTED                   (0xc0000241)
#define STATUS_BAD_COMPRESSION_BUFFER               (0xc0000242)
#define STATUS_USER_MAPPED_FILE                     (0xc0000243)
#define STATUS_AUDIT_FAILED                         (0xc0000244)
#define STATUS_TIMER_RESOLUTION_NOT_SET             (0xc0000245)
#define STATUS_CONNECTION_COUNT_LIMIT               (0xc0000246)
#define STATUS_LOGIN_TIME_RESTRICTION               (0xc0000247)
#define STATUS_LOGIN_WKSTA_RESTRICTION              (0xc0000248)
#define STATUS_IMAGE_MP_UP_MISMATCH                 (0xc0000249)

#define STATUS_INSUFFICIENT_LOGON_INFO              (0xc0000250)
#define STATUS_BAD_DLL_ENTRYPOINT                   (0xc0000251)
#define STATUS_BAD_SERVICE_ENTRYPOINT               (0xc0000252)
#define STATUS_LPC_REPLY_LOST                       (0xc0000253)
#define STATUS_IP_ADDRESS_CONFLICT1                 (0xc0000254)
#define STATUS_IP_ADDRESS_CONFLICT2                 (0xc0000255)
#define STATUS_REGISTRY_QUOTA_LIMIT                 (0xc0000256)
#define STATUS_PATH_NOT_COVERED                     (0xc0000257)
#define STATUS_NO_CALLBACK_ACTIVE                   (0xc0000258)
#define STATUS_LICENSE_QUOTA_EXCEEDED               (0xc0000259)
#define STATUS_PWD_TOO_SHORT                        (0xc000025a)
#define STATUS_PWD_TOO_RECENT                       (0xc000025b)
#define STATUS_PWD_HISTORY_CONFLICT                 (0xc000025c)
#define STATUS_PLUGPLAY_NO_DEVICE                   (0xc000025e)
#define STATUS_UNSUPPORTED_COMPRESSION              (0xc000025f)
#define STATUS_INVALID_HW_PROFILE                   (0xc0000260)
#define STATUS_INVALID_PLUGPLAY_DEVICE_PATH         (0xc0000261)
#define STATUS_DRIVER_ORDINAL_NOT_FOUND             (0xc0000262)
#define STATUS_DRIVER_ENTRYPOINT_NOT_FOUND          (0xc0000263)
#define STATUS_RESOURCE_NOT_OWNED                   (0xc0000264)
#define STATUS_TOO_MANY_LINKS                       (0xc0000265)
#define STATUS_QUOTA_LIST_INCONSISTENT              (0xc0000266)
#define STATUS_FILE_IS_OFFLINE                      (0xc0000267)
#define STATUS_EVALUATION_EXPIRATION                (0xc0000268)
#define STATUS_ILLEGAL_DLL_RELOCATION               (0xc0000269)
#define STATUS_LICENSE_VIOLATION                    (0xc000026a)
#define STATUS_DLL_INIT_FAILED_LOGOFF               (0xc000026b)
#define STATUS_DRIVER_UNABLE_TO_LOAD                (0xc000026c)
#define STATUS_DFS_UNAVAILABLE                      (0xc000026d)
#define STATUS_VOLUME_DISMOUNTED                    (0xc000026e)
#define STATUS_WX86_INTERNAL_ERROR                  (0xc000026f)
#define STATUS_WX86_FLOAT_STACK_CHECK               (0xc0000270)
#define STATUS_VALIDATE_CONTINUE                    (0xc0000271)
#define STATUS_NO_MATCH                             (0xc0000272)
#define STATUS_NO_MORE_MATCHES                      (0xc0000273)
#define STATUS_INVALID_VLM_OPERATION                (0xc0000274)
#define STATUS_NOT_A_REPARSE_POINT                  (0xc0000275)
#define STATUS_IO_REPARSE_TAG_INVALID               (0xc0000276)
#define STATUS_IO_REPARSE_TAG_MISMATCH              (0xc0000277)
#define STATUS_IO_REPARSE_DATA_INVALID              (0xc0000278)
#define STATUS_IO_REPARSE_TAG_NOT_HANDLED           (0xc0000279)


#define STATUS_SAM_INIT_FAILURE                     (0xC00002CB)


#define RPC_NT_INVALID_STRING_BINDING               (0xC0020001)
#define RPC_NT_WRONG_KIND_OF_BINDING                (0xC0020002)
#define RPC_NT_INVALID_BINDING                      (0xC0020003)
#define RPC_NT_PROTSEQ_NOT_SUPPORTED                (0xC0020004)
#define RPC_NT_INVALID_RPC_PROTSEQ                  (0xC0020005)
#define RPC_NT_INVALID_STRING_UUID                  (0xC0020006)
#define RPC_NT_INVALID_ENDPOINT_FORMAT              (0xC0020007)
#define RPC_NT_INVALID_NET_ADDR                     (0xC0020008)
#define RPC_NT_NO_ENDPOINT_FOUND                    (0xC0020009)
#define RPC_NT_INVALID_TIMEOUT                      (0xC002000A)
#define RPC_NT_OBJECT_NOT_FOUND                     (0xC002000B)
#define RPC_NT_ALREADY_REGISTERED                   (0xC002000C)
#define RPC_NT_TYPE_ALREADY_REGISTERED              (0xC002000D)
#define RPC_NT_ALREADY_LISTENING                    (0xC002000E)
#define RPC_NT_NO_PROTSEQS_REGISTERED               (0xC002000F)
#define RPC_NT_NOT_LISTENING                        (0xC0020010)
#define RPC_NT_UNKNOWN_MGR_TYPE                     (0xC0020011)
#define RPC_NT_UNKNOWN_IF                           (0xC0020012)
#define RPC_NT_NO_BINDINGS                          (0xC0020013)
#define RPC_NT_NO_PROTSEQS                          (0xC0020014)
#define RPC_NT_CANT_CREATE_ENDPOINT                 (0xC0020015)
#define RPC_NT_OUT_OF_RESOURCES                     (0xC0020016)
#define RPC_NT_SERVER_UNAVAILABLE                   (0xC0020017)
#define RPC_NT_SERVER_TOO_BUSY                      (0xC0020018)
#define RPC_NT_INVALID_NETWORK_OPTIONS              (0xC0020019)
#define RPC_NT_NO_CALL_ACTIVE                       (0xC002001A)
#define RPC_NT_CALL_FAILED                          (0xC002001B)
#define RPC_NT_CALL_FAILED_DNE                      (0xC002001C)
#define RPC_NT_PROTOCOL_ERROR                       (0xC002001D)


#define RPC_NT_SS_IN_NULL_CONTEXT                   (0xC0030004)


/* FIXME: Are these official values ?? */
#define STATUS_FS_QUERY_REQUIRED                    (0xC1000001)
#define STATUS_HANDLE_NOT_WAITABLE                  (0xC1000002)
#define STATUS_OBJECT_FILE_MISMATCH                 (0xC1000003)
#define STATUS_INVALID_PARAMETER_MAX                (0xC1000004)
#define STATUS_CONFLICTING_ADDRESS                  (0xC1000005)
#define STATUS_NO_MEDIA_IN_DRIVE                    (0xC1000006)

#endif /* __INCLUDE_DDK_STATUS_H */






